@@ -60,6 +60,13 @@ Image
 		(*anyimage)->colormap = (RGB *)NULL;
 	}
 
+
+	if ((*anyimage)->alpha_color != (RGB *) NULL)
+	{
+		(void) free ((char *) (*anyimage)->alpha_color);
+		(*anyimage)->alpha_color = (RGB *)NULL;
+	}
+
 	if ((*anyimage)->signature != (char *) NULL)
 	{
 		(void) free ((char *) (*anyimage)->signature);
