@@ -2,8 +2,8 @@
  *  WWW document access counter
  *
  *  RCS:
- *      $Revision: 2.3 $
- *      $Date: 1996/05/03 02:20:22 $
+ *      $Revision: 1.2 $
+ *      $Date: 2001/04/08 16:15:51 $
  *
  *  Security:
  *      Unclassified
@@ -93,24 +93,27 @@ char
         use_st=False,
         ignore_site=False;
 
+#ifdef SYS_WIN32
+	char
+		*script_name=NULL,
+	    drive[16];
+
+#endif /* SYS_WIN32 */
 
     char
-        *script_name,
-        drive[16],
         hour[10],
         min[10],
         ampm[10],
+        *s,
         *dbuf=(char *) NULL,
         *browser_type,
+        *penv,
         *remote_ip,
         *query_string,
         filename[MaxTextLength],
         tmpbuf[MaxTextLength],
         digitbuf[MaxTextLength];
 
-    Sllist
-        *llp;
-
     DigitInfo
         digit_info;
 
@@ -120,14 +123,7 @@ char
 /*
 **---------initialize globals------Starts---
 */
-    g_refhost=(Sllist *) NULL;
-    g_ighost=(Sllist *) NULL;
-
-    Grhost=0;
-    Gsite=0;
     Gdebug=False;
-    Gauto_file_creation=0;
-    Gstrict_mode=1;
     GrgbMappingIsError=1;
 
 #ifdef SYS_WIN32
@@ -151,6 +147,7 @@ char
     left_pad=max_digits;
     use_st=False;
     display_what=SHOW_COUNTER;
+    penv=(char *) NULL;
     remote_ip=(char *) NULL;
 
 
@@ -167,6 +164,12 @@ char
             Gdebug=True;
             break;
         }
+        s="-version";
+        if(!strncmp(argv[i],s,(int) strlen(s)))
+        {
+            (void) fprintf(stderr,"Count version %s\n",IMAGE_COMMENT);
+            exit(1);
+        }
     }
 
 
@@ -205,13 +208,19 @@ char
 
     /* get the remote IP, we'll need it in many places */
 #ifdef __VMS
-    remote_ip = (char *) getenv(SYM_PREFIX "REMOTE_ADDR");
+    penv=(char *) getenv(SYM_PREFIX "HTTP_X_FORWARDED_FOR");
+    if (penv == NULL)
+        penv = (char *) getenv(SYM_PREFIX "REMOTE_ADDR");
 #else
-    remote_ip = (char *) getenv("REMOTE_ADDR");
+    penv=(char *) getenv("HTTP_X_FORWARDED_FOR");
+    if (penv == NULL)
+        penv = (char *) getenv("REMOTE_ADDR");
 #endif
 
-    if (remote_ip == (char *) NULL)
+    if (penv == (char *) NULL)
         remote_ip="0.0.0.0";
+    else
+        remote_ip=mystrdup(penv);
  
     Debug2("[%s] Count %s: --Debug--",GetTime(),Version);
 
@@ -222,21 +231,19 @@ char
         safeStrcpy(tmpbuf,"CONFIG_DIR,DIGIT_DIR,DATA_DIR,LOG_DIR must differ",
                 sizeof(tmpbuf)-1);
         Warning(tmpbuf);
-        PrintHeader();
+
         StringImage(tmpbuf);
         exit(1);
     }
 
     /*
-    ** parse the authorization list
+    ** parse the config file
     */
     ParseConfig ();
 
+
+
     /* print what we read, for debug only....... Remore asap */
-    Debug2("---- g_refhost linked list -----",0,0);
-    printList(g_refhost);
-    Debug2("---- g_ighost linked list -----",0,0);
-    printList(g_ighost);
 
     /*
     ** we will be only be here if everything is fine, otherwise we'll be
@@ -244,21 +251,17 @@ char
     */
 
 
-     Debug2("Gsite: %d",Gsite,0);
-     Debug2("Gauto_file_creation= %d",Gauto_file_creation,0);
-     Debug2("Gstrict_mode= %d",Gstrict_mode,0);
 
-#ifdef ACCESS_AUTH
     /*
     ** check if the referer is a remote host or not. refere should
     ** be the local host.
     */
-    ihas_referer=authRemhost();
-    Debug2(">>>>    ihas_referer=%d",ihas_referer,0);
-    
+    if (getcfgStrictMode() == True)
+    {
+        ihas_referer=authRemhost();
+    } 
     /* if not authorized, we won't be here */
 
-#endif /* ACCESS_AUTH */
  
 
    /*
@@ -275,7 +278,6 @@ char
         *tmpbuf='\0';
         safeStrcpy(tmpbuf,"Empty QUERY_STRING",sizeof(tmpbuf)-1);
         Warning(tmpbuf);
-        PrintHeader ();
         StringImage(tmpbuf);
         exit(1);
     }
@@ -290,11 +292,15 @@ char
     if (rc)
     {
         Debug2("Error parsing QUERY_STIRNG",0,0);
-        PrintHeader();
+
         StringImage("Error parsing QUERY_STRING");
         exit(0);
     }
 
+    setUserReload(digit_info.user_reload);
+
+#if 0
+
     Debug2("\n",0,0);
     Debug2("Parsed QUERY_STRING",0,0); 
     Debug2(" ft=%d",frame_info.width,0);
@@ -321,27 +327,42 @@ char
         digit_info.width, digit_info.height);
     Debug2(" md=%d", digit_info.maxdigits,0);
     Debug2(" pad=%d", digit_info.leftpad,0);
-    Debug2(" ddhead=%s",digit_info.ddhead,0);
+    Debug2(" dd=%s",digit_info.ddhead,0);
     Debug2(" st=%d",digit_info.st,0);
     Debug2(" sh=%d", digit_info.show,0);
     Debug2(" df=%s",digit_info.datafile,0);
     Debug2(" lit=%s",digit_info.literal,0);
     Debug2(" incr=%d",digit_info.increment_counter,0);
     Debug2(" negate=%d",digit_info.negate,0);
+    Debug2(" user_reload=%d",digit_info.user_reload,0);
     Debug2(" rotate=%d",digit_info.rotate,0);
     Debug2(" degrees=%d",digit_info.rotate_degrees,0);
     Debug2(" timezone=%s",digit_info.time_z,0);
     Debug2(" timeformat=%d",digit_info.time_format,0);
     Debug2(" istrip=%d",digit_info.use_strip,0);
     Debug2(" image=%s", digit_info.gif_file,0);
+    Debug2(" sfd=%d",digit_info.show_from_date,0);
     Debug2(" parsed QS--\n",0,0);
+#endif /* if 0 */
+
 
     /*
     ** now set some values for future retrieval
     */
+    /*
+    ** well, cache is not used anymore.. it's there just for backward
+    ** compatibility. Only use "expires". Using cache and expires together
+    ** creates too much confusion. If you to tell the browser not to cache
+    ** specify expires=0, speicfy a value greater than 0 if want the browser
+    ** to cache for that amount of seconds.
+    ** muquit@muquit.com Sun Mar 25 20:01:46 EST 2001 
+    */
+/*
     setCache(digit_info.cache);
+*/
     setExpire(digit_info.expires);
 
+
     /*
     ** check if digit head has any path in it
     ** a bad bug, reported by Ojanpera <ojapa@japo.fi>
@@ -352,7 +373,7 @@ char
         *tmpbuf='\0';
         safeStrcpy(tmpbuf,"Illegal digit path!",sizeof(tmpbuf)-1);
         Warning(tmpbuf);
-        PrintHeader ();
+
         StringImage(tmpbuf);
         exit(0);
     }
@@ -381,7 +402,7 @@ char
             *tmpbuf='\0';
             safeStrcpy(tmpbuf,"Illegal datafile path!",sizeof(tmpbuf)-1);
             Warning(tmpbuf);
-            PrintHeader ();
+
             StringImage(tmpbuf);
             exit(1);
         }
@@ -405,40 +426,15 @@ char
             /*
             ** put a dummy here
             */
-            remote_ip = "dummy_ip";
+            remote_ip = mystrdup("dummy_ip");
         }
         else
         {
-            for (llp=g_ighost;llp;llp=llp->next)
+            ignore_site=ignoreIP(remote_ip);
+            if (ignore_site == True)
             {
-                ignore_site=CheckRemoteIP(remote_ip,llp->item);
-                if (ignore_site == True)
-                {
-                    Debug2("Not counting IP: %s",remote_ip,0);
-                    Debug2("ignored mask: %s",llp->item,0);
-
-                    /*
-                    ** break was missing, which making this check useless
-                    ** in most cases.
-                    ** Thanks to katz@greco2.polytechnique.fr (Armand Katz)
-                    ** Nov-26-1997
-                    */
-                    break;
-                }
+                Debug2("ignoring site: %s",remote_ip,0);
             }
-
-            /*
-            for (i=0; i < Gsite; i++)
-            {
-                ignore_site=CheckRemoteIP(remote_ip,GignoreSite[i]);
-                if (ignore_site == True)
-                {
-                    Debug2("Not counting IP: %s",remote_ip,0);
-                    Debug2("GignoreSite[%d]:%s",i,GignoreSite[i]);
-                    break;
-                }
-            }
-            */
         }
     }
 
@@ -447,13 +443,7 @@ char
     */
 
 
-    if (display_what == SHOW_COUNTER)
-    {
-        Debug2("Maxdigits=%d",max_digits,0);
-    }
-
-#ifdef ACCESS_AUTH 
-    if (Gstrict_mode == 1)
+    if (getcfgStrictMode() == True)
     {
         if (ihas_referer == 0)
         {
@@ -466,14 +456,14 @@ char
             ** broken browser like this will see a literal string
             */
             (void) strcpy(digit_info.literal,"888888");
-
-            safeStrcpy(digitbuf,"888888",7);
+            strcpy(digitbuf,"888888");
             /*
             ** if gif file was asked to display,we'll be in trouble
             */
             initImageParams(&digit_info,&frame_info);
             digit_info.Frame=False;
-            WriteCounterImage(digitbuf,&digit_info,&frame_info);
+            digit_info.pdigits=mystrdup(digitbuf);
+            makeCounterImage(&digit_info,&frame_info);
 
             /*
             ** we won't be here
@@ -492,32 +482,54 @@ char
         ** try to get the browser name, it will give us a good
         ** feedback.
         */
-#ifdef DO_LOG
+            if (getcfgLogErrmsg() == True)
+            {
 
 #ifdef __VMS
-            browser_type=(char *) getenv(SYM_PREFIX "HTTP_USER_AGENT");
+                browser_type=(char *) getenv(SYM_PREFIX "HTTP_USER_AGENT");
 #else
-            browser_type=(char *) getenv("HTTP_USER_AGENT");
+                browser_type=(char *) getenv("HTTP_USER_AGENT");
 #endif
-            if ((browser_type != (char *) NULL) &&
-                (*browser_type != '\0'))
-            {
-                if (remote_ip != (char *) NULL)
+                if ((browser_type != (char *) NULL) &&
+                    (*browser_type != '\0'))
                 {
-                    *tmpbuf='\0';
-                    safeStrcpy(tmpbuf,browser_type,sizeof(tmpbuf)-1);
-                    safeStrcpy(digitbuf,"browser without referer ",
-                            sizeof(digitbuf)-1);
-                    safeStrcat(digitbuf,tmpbuf,sizeof(digitbuf)-1,
-                            sizeof(digitbuf),strlen(digitbuf));
-                    Warning(digitbuf);
-                    *digitbuf='\0';
+                    if (remote_ip != (char *) NULL)
+                    {
+                        *tmpbuf='\0';
+                        safeStrcpy(tmpbuf,browser_type,sizeof(tmpbuf)-1);
+                        safeStrcpy(digitbuf,"browser without referer ",
+                                sizeof(digitbuf)-1);
+                        safeStrcat(digitbuf,tmpbuf,sizeof(digitbuf)-1,
+                                sizeof(digitbuf),strlen(digitbuf));
+                        Warning(digitbuf);
+                        *digitbuf='\0';
+                    }
                 }
             }
-#endif  /* DO_LOG */
         }
-    }   /* Gstrict_mode == 1 */
-#endif
+    }   /*getcfgStringMode()  == True */
+
+    /*
+    ** if gif file is specified, make the full path
+    */
+    if (*digit_info.gif_file != '\0')
+    {
+        
+        /*
+        ** security fix. oct-10-1997
+        ** first check if the image file specified with image=
+        ** parameter has any path in it
+        */
+        if (checkfilename(digit_info.gif_file) != 0)
+        {
+            StringImage("Illegal image filename!");
+            exit(0);
+        }
+
+        makeGIFPath(tmpbuf,sizeof(tmpbuf),g_sZ_ibase,&digit_info);
+        /* overwrite digit_info.gif_file */
+        safeStrcpy(digit_info.gif_file,tmpbuf,1023);
+    }
 
     switch (display_what)
     {
@@ -541,26 +553,25 @@ char
             break;
         }
 
+        case SHOW_COUNTDOWN:
+        {
+            displayCountdown(&digit_info,&frame_info);
+            break;
+        }
+        case SHOW_VERSION:
+        {
+            multilineStringImage(IMAGE_COMMENT,ALIGN_CENTER,0,0,0,0,255,0);
+            exit(0);
+            break;
+        }
+
         case SHOW_GIF_FILE:
         {
             if (*digit_info.gif_file == '\0')
             {
-                PrintHeader();
                 StringImage("No GIF file specified to display");
                 exit(1);
             }
-            /*
-            ** security fix. oct-10-1997
-            ** first check if the image file specified with image=
-            ** parameter has any path in it
-            */
-            if (checkfilename(digit_info.gif_file) != 0)
-            {
-                PrintHeader();
-                StringImage("Illegal image filename!");
-                exit(0);
-            }
-
             /* 
             ** check if counter is needed to be incremented
             */
@@ -584,7 +595,6 @@ char
                     /* check if some idiot is trying to screw up */
                     if (checkfilename(digit_info.datafile))
                     {
-                        PrintHeader();
                         StringImage("Illegal character in filename!");
                         exit(0);
                     }
@@ -604,53 +614,42 @@ char
 
                     Debug2("datafile path=%s",filename,0);
                     if (remote_ip)
-                    {
-                        for (llp=g_ighost;llp;llp=llp->next)
-                            ignore_site=CheckRemoteIP(remote_ip,llp->item);
-                    }
+                        ignore_site=ignoreIP(remote_ip);
+                    /* 
+                    ** NOTE: dbuf is malloc'd but we won't care to free it
+                    ** as memory will be freed when the CGI exits
+                    */
                     dbuf=rwCounterDataFile(filename,
                             remote_ip,ignore_site,False,
                             1,digit_info.increment_counter);
 
-                    if (dbuf)
-                        (void) free(dbuf);
+                }
+
+                /*
+                ** if literal string is specified, it might be later used
+                ** composite. mm, jan-18-1999
+                */
+                if ((int)strlen(digit_info.literal) > 0)
+                {
+                    dbuf=mystrdup(digit_info.literal);
                 }
             }
 
-            /* make the gif filepath*/
-            safeStrcpy(digitbuf,g_sZ_ibase,sizeof(digitbuf)-1);
-#ifndef __VMS
-            safeStrcat(digitbuf,"/",sizeof(digitbuf)-1,
-                    sizeof(digitbuf),strlen(digitbuf));
-#endif
-
-            safeStrcat(digitbuf,digit_info.ddhead,sizeof(digitbuf)-1,
-                    sizeof(digitbuf),strlen(digitbuf));
-#ifndef __VMS
-            safeStrcat(digitbuf,"/",sizeof(digitbuf)-1,sizeof(digitbuf),
-                    strlen(digitbuf));
-#else
-/*          For VMS add a ] before adding name of .gif file  */
-            safeStrcat(digitbuf,"]",sizeof(digitbuf)-1,sizeof(digitbuf),
-                    strlen(digitbuf));
-#endif
-            safeStrcat(digitbuf,digit_info.gif_file,sizeof(digitbuf)-1,
-                    sizeof(digitbuf),strlen(digitbuf));
-
-
-            /* Next bit commented out but add VMS mod just in case */
-            /*
-#ifndef __VMS
-            (void) sprintf(digitbuf,"%s/%s/%s",
-#else
-            (void) sprintf(digitbuf,"%s%s]%s",
-#endif
-                DIGIT_DIR,digit_info.ddhead,digit_info.gif_file);
-            */
-            WriteCounterImage(digitbuf,&digit_info,&frame_info);
-            /*
-            ** we won't be here
+           /*
+            ** we will do some bad thing here. the gif_file member of
+            ** the DigitInfo structure is not used anymore. so we will
+            ** put the counter digit buffer into it. Just for my own
+            ** future reference
+            ** mm, dec-19-1998
             */
+            if (dbuf != NULL)
+                digit_info.pdigits=mystrdup(dbuf);
+            else
+                digit_info.pdigits=mystrdup(digitbuf);
+
+            makeCounterImage(&digit_info,&frame_info);
+            /* we won't be here */
+
             break;
         }
         case SHOW_COUNTER:
@@ -660,7 +659,6 @@ char
             if (digit_info.comma == True)
                 digit_info.leftpad=False;
 
-            Debug2("digit_info.datafile=\"%s\"",digit_info.datafile,0);
             if ((int) strlen(digit_info.literal) > 0)
             {
                 /* digitbuf buffer must be larger than MAX_DIGITS */
@@ -684,7 +682,6 @@ char
                 dbuf=mystrdup(digitbuf);
                 if (dbuf == (char *) NULL)
                 {
-                    PrintHeader();
                     StringImage("memory allocation problem");
                     exit(0);
                  }
@@ -717,7 +714,6 @@ char
               dbuf=mystrdup(digitbuf);
               if (dbuf == (char *) NULL)
               {
-                  PrintHeader();
                   StringImage("memory allocation problem");
                   exit(0);
               }
@@ -736,18 +732,15 @@ char
                 safeStrcat (filename,digit_info.datafile,sizeof(filename)-1,
                         sizeof(filename),strlen(filename));
 
-                Debug2("datafile== %s",filename,0);
-
                 /*
                 ** check if the counter file exists or not
                 */
-                if (Gauto_file_creation == 0)
+                if (getcfgAutoFileCreation() == False)
                 {
                     if (CheckFile (filename) != 0)
                     {
                         if (strlen(filename) > sizeof(tmpbuf)-60)
                         {
-                            PrintHeader();
                             StringImage("buffer overflow in filename");
                             exit(0);
                         }
@@ -755,7 +748,6 @@ char
                         (void) sprintf (tmpbuf,
                     "Counter datafile \"%s\" must be created first!", filename);
                         Warning(tmpbuf);
-                        PrintHeader ();
                         StringImage (tmpbuf);
                         exit(1);
                     }
@@ -789,7 +781,6 @@ char
     }
     else
     {
-        PrintHeader();
         StringImage("unknown problem!");
         exit(0);
     }
@@ -804,7 +795,6 @@ char
             image=CreateBaseImage(1,1,0,0,0,DirectClass);
             if (image == (Image *) NULL)
             {
-                PrintHeader();
                 StringImage("Failed to create 1x1 GIF image");
                 exit(1);
             }
@@ -818,7 +808,8 @@ char
 
        if (digit_info.leftpad == False)    /* no left padding */
        {
-            WriteCounterImage(digitbuf,&digit_info,&frame_info);
+            digit_info.pdigits=mystrdup(digitbuf);
+            makeCounterImage(&digit_info,&frame_info);
             /*
             ** we will not be here, we'll exit from LoadDigits()
             */
@@ -830,15 +821,11 @@ char
             else
                 left_pad=0;
 
-            Debug2(" MaxDigits=%d",max_digits,0);
-            Debug2(" left_pad=%d",left_pad,0);
-
               if ((left_pad < max_digits) && (left_pad != 0))
               {
 
                 if (left_pad > sizeof(tmpbuf)-1)
                 {
-                    PrintHeader();
                     StringImage("buffer overflow detected in left_pad");
                     exit(0);
                 }
@@ -848,15 +835,16 @@ char
                     (void) sprintf(tmpbuf,"%s0",tmpbuf);
                 }
                 (void) sprintf(tmpbuf,"%s%s",tmpbuf,digitbuf);
-                WriteCounterImage(tmpbuf,&digit_info,&frame_info);
+                digit_info.pdigits=mystrdup(tmpbuf);
+                makeCounterImage(&digit_info,&frame_info);
                 /*
                 ** we will not be here
                 */
               }
               else    /* MaxDigits*/
               {
-                  Debug2(" We are in MaxDigits=%d",max_digits,0);
-                  WriteCounterImage(digitbuf,&digit_info,&frame_info);
+                  digit_info.pdigits=mystrdup(digitbuf);
+                  makeCounterImage(&digit_info,&frame_info);
                 /*
                 ** we won't be here
                 */
