@@ -17,235 +17,14 @@
 #endif
 
 /* private protos */
-static void printConfigError _Declare ((int err_type));
-
-void ParseConfig ()
-{
-    FILE
-        *fp;
-
-    int
-        i;
-
-    char
-        buf[MaxLineLength+1];
-
-    i=0;
-
-    *buf='\0';
-
-#ifdef SYS_WIN32
-
-        /*
-        ** g_sZ_vbase has a trailing \ at the end
-        */
-        safeStrcpy(buf,g_sZ_vbase,sizeof(buf)-1);
-        safeStrcat(buf,CONFIG_DIR,sizeof(buf)-1,sizeof(buf),strlen(buf));
-        safeStrcat(buf,"/",sizeof(buf)-1,sizeof(buf),strlen(buf));
-        safeStrcat(buf,CONFIG_FILE,sizeof(buf)-1,sizeof(buf),strlen(buf));
+static int  ParseCountdown   _Declare ((char *given,int *t_year,
+                                        int *t_month,int *t_day,int *t_hr,
+                                        int *t_min,int *t_sec));
+static int ParseTwoint       _Declare  ((char *given,
+                                        int *x,
+                                        int *y));
 
 
-#else
-
-    safeStrcpy(buf,CONFIG_DIR,sizeof(buf)-1);
-    safeStrcat(buf,DIR_SEPARATOR,sizeof(buf)-1,sizeof(buf),strlen(buf));
-    safeStrcat(buf,CONFIG_FILE,sizeof(buf)-1,sizeof(buf),strlen(buf));
-#endif
-
-    fp = fopen(buf, "r");
-    if (fp == (FILE *) NULL)
-    {
-        printConfigError(ConfigOpenFailed);
-        /* we won't be here */
-    }
-
-    /*
-    ** first block: automatic file creation. If the block contains 1,
-    ** automatic file creation is allowed. if 0, not allowed.
-    ** automatic file creation is a bad thing, but some sites may want
-    ** it because they are running their server behind a firewall and
-    ** they trust their users
-    ** BLOCK 1
-    */
-
-    *buf='\0';
-    (void) GetLine(fp, buf);
-    if (strcmp(buf, "{") != 0)
-    {
-        (void) fclose ((FILE *) fp);
-        printConfigError(NoAutofcBlock);
-        /* we won't be here */
-    }
-    while (True)
-    {
-        if (!GetLine (fp, buf))
-        {
-            (void) fclose ((FILE *) fp);
-             printConfigError(UnpexpectedEof);                       
-             /* we won't be here */
-        }
-        if (strcmp(buf, "}") == 0)
-            break;
-        RemoveTrailingSp (buf);
-        Gauto_file_creation=atoi(buf);
-    }
-
-    /*
-    ** strict mode block
-    ** BLOCK 2
-    */
-    *buf='\0';
-    (void) GetLine(fp, buf);
-    if (strcmp(buf, "{") != 0)
-    {
-        (void) fclose ((FILE *) fp);
-        printConfigError(NoStrictBlock);
-        /* we won't be here */
-    }
-    while (True)
-    {
-        if (!GetLine (fp, buf))
-        {
-            (void) fclose ((FILE *) fp);
-             printConfigError(UnpexpectedEof);                       
-            /* we won't be here */
-        }
-        if (strcmp(buf, "}") == 0)
-            break;
-        RemoveTrailingSp (buf);
-        Gstrict_mode=atoi(buf);
-    }
-
-    /*
-    ** rgb coloname lookup 
-    ** BLOCK 3
-    */
-    *buf='\0';
-    (void) GetLine(fp, buf);
-    if (strcmp(buf, "{") != 0)
-    {
-        (void) fclose ((FILE *) fp);
-        printConfigError(NoRgbMappingBlock);
-        /* we won't be here */
-    }
-    while (True)
-    {
-        if (!GetLine (fp, buf))
-        {
-            (void) fclose ((FILE *) fp);
-             printConfigError(UnpexpectedEof);                       
-             /* we won't be here */
-        }
-        if (strcmp(buf, "}") == 0)
-            break;
-        RemoveTrailingSp (buf);
-        GrgbMappingIsError=atoi(buf);
-    }
-
-    /*
-    ** ignore count block
-    ** BLOCK 4
-    */
-    *buf='\0';
-    (void) GetLine (fp, buf);
-    if (strcmp (buf, "{") != 0)
-    {
-        (void) fclose ((FILE *) fp);
-        printConfigError(NoIgnoreHostsBlock);
-        /* we won't be here */
-    }
-
-    while (True)
-    {
-        if (!GetLine (fp, buf))
-        {
-            (void) fclose ((FILE *) fp);
-            printConfigError(UnpexpectedEof);
-            /* we won't be here */
-        }
-
-        if (strcmp(buf, "}") == 0)
-            break;
-        RemoveTrailingSp (buf);
-        /* g_ighost is initialized to NULL in main.c, it's required */
-        Gsite++;
-        additemToList(&g_ighost,buf);
-    }
-
-        if (Gsite > 0)
-        {
-            Debug2("Ignore Hosts:",0,0);
-        }
-        else
-        {
-            Debug2("Will count access from any host",0,0);
-        }
-        /*
-        for (i=0; i < Gsite; i++)
-        {
-            Debug2("Site[%d] %s",i,GignoreSite[i]);
-        }
-        */
-
-#ifdef ACCESS_AUTH
-    (void) GetLine (fp, buf);
-    if (strcmp(buf, "{") != 0)
-    {
-       (void) fclose ((FILE *) fp);
-        printConfigError(NoRefhBlock);
-        /* we won't be here */
-    }
-    while (True)
-    {
-        if (!GetLine (fp, buf))
-        {
-            (void) fclose ((FILE *) fp);
-            printConfigError(UnpexpectedEof);
-            /* we won't be here */
-        }
-        if (strcmp (buf, "}") == 0)
-            break;
-        RemoveTrailingSp (buf);
-        /*
-        GrefererHost[Grhost++]= mystrdup(buf);
-        */
-
-        /* g_refhost is initialized to NULL at main.c, it's essential */
-        Grhost++;
-        additemToList(&g_refhost,buf);
-
-    }
-
-        if (Grhost > 0)
-        {
-            Debug2("-Referer Hosts-",0,0);
-        }
-        else
-        {
-            Debug2("Grhost: %d", Grhost,0);
-        }
-        /*
-        for (i=0; i < Grhost; i++)
-        {
-            Debug2("Referer[%d] %s",i,GrefererHost[i]);
-        }
-        */
-
-#endif /* ACCESS_AUTH */
-    (void) fclose ((FILE *) fp);
-}
-
-#ifdef TEST
-
-void main (argc, argv)
-int
-    argc;
-char
-    **argv;
-{
-    ParseAuthorizationList ();
-}
-#endif /* TEST */
 
 
 /*
@@ -432,26 +211,36 @@ char
 int
     *sets;
 {
-   if (mystrcasecmp(given,"counter") == 0)
+   if (mystrncasecmp(given,"counter",6) == 0)
    {
         *sets=SHOW_COUNTER;
         return (True);
    } 
-   else if (mystrcasecmp(given,"clock") == 0)
+   else if (mystrncasecmp(given,"clock",2) == 0)
    {
         *sets=SHOW_CLOCK;
         return (True);
    }
-   else if (mystrcasecmp(given,"date") == 0)
+   else if (mystrncasecmp(given,"date",2) == 0)
    {
         *sets=SHOW_DATE;
         return (True);
    }
-   else if (mystrcasecmp(given,"image") == 0)
+   else if (mystrncasecmp(given,"image",1) == 0)
    {
         *sets=SHOW_GIF_FILE;
         return (True);
    }
+   else if (mystrncasecmp(given,"countdown",6) == 0)
+   {
+       *sets=SHOW_COUNTDOWN;
+       return(True);
+   }
+   else if (mystrncasecmp(given,"version",3) == 0)
+   {
+       *sets=SHOW_VERSION;
+       return(True);
+   }
    else
         return (False);
 
@@ -464,32 +253,36 @@ char
 int
     *sets;
 {
-    if (mystrcasecmp(given,"MMDDYY") == 0)
+    if (mystrncasecmp(given,"MMDDYY",6) == 0)
     {
        *sets=DATE_MMDDYY;
        return(True); 
     }
-    else if (mystrcasecmp(given,"DDMMYY") == 0)
+    else if (mystrncasecmp(given,"DDMMYY",6) == 0)
     {
        *sets=DATE_DDMMYY;
        return(True); 
     }
-    else if (mystrcasecmp(given,"YYMMDD") == 0)
+    else if ((mystrncasecmp(given, "YYMMDD",6) == 0) ||
+             (mystrncasecmp(given, "YYYYMMDD", 8) == 0))
     {
        *sets=DATE_YYMMDD;
        return(True); 
     }
-    else if (mystrcasecmp(given,"YYDDMM") == 0)
+    else if ((mystrncasecmp(given,"YYDDMM",6) == 0) ||
+             (mystrncasecmp(given,"YYYYDDMM", 8) == 0))
     {
        *sets=DATE_YYDDMM;
        return(True); 
     }
-    else if (mystrcasecmp(given,"MMYYDD") == 0)
+    else if ((mystrncasecmp(given,"MMYYDD",6) == 0) ||
+             (mystrncasecmp(given,"MMYYYYDD", 8) == 0))
     {
        *sets=DATE_MMYYDD;
        return(True); 
     }
-    else if (mystrcasecmp(given,"DDYYMM") == 0)
+    else if ((mystrncasecmp(given,"DDYYMM",6) == 0) ||
+             (mystrncasecmp(given, "DDYYYYMM", 8) == 0))
     {
        *sets=DATE_MMYYDD;
        return(True); 
@@ -595,11 +388,7 @@ unsigned char
                                red, green, blue, rgbname, RGB_MAPPING_DICT);
                         if (GrgbMappingIsError == 0)
                         {
-                            /*
-                            sprintf(emsg, "Please specify \"%d;%d;%d\" instead of \"%s\" for color.", red, green, blue, rgbname);
-                            */
                             (void) sprintf(emsg,"Please specify \"%02x%02x%02x\" instead of \"%.100s\" for color.",red,green,blue,rgbname);
-                            PrintHeader();
                             StringImage(emsg);
                         }
                         else
@@ -625,23 +414,112 @@ unsigned char
     return(True);
 }
 
-int ParseSize(given,set_w,set_h )
+
+/*
+**  ParseCountdown()
+**  parses a delemited string and breaks it up
+**
+**  Parameters:
+**  char    *given      string to parse. the items in the strings are
+**                      delemeted by ; or -
+**  int     *t_year     target year  - returns;
+**  int     *t_month    target month  - returns;
+**  int     *t_day      target day  - returns;
+**  int     *t_hr       target hour - returns;
+**  int     *t_min      target min - returns;
+**  int     *t_sec      target sec - returns;
+**
+**  Return Values:
+**  True if everything is ok, False otherwise
+**
+**  Limitations and Comments:
+**  The year can be a past or future year. if past, all values will be
+**  negative
+**
+**  Development History:
+**      who                  when           why
+**      ma_muquit@fccc.edu   Dec-13-1998    first cut
+**                           Jul-17-1999    added sanity check
+*/
+
+
+static int ParseCountdown(given,t_year,t_month,t_day,t_hr,t_min,t_sec)
+char
+    *given; 
+int
+    *t_year,
+    *t_month,
+    *t_day,
+    *t_hr,
+    *t_min,
+    *t_sec;
+{
+
+
+    if ((sscanf(given,"%d;%d;%d;%d;%d;%d",
+                    t_year,
+                    t_month,
+                    t_day,
+                    t_hr,
+                    t_min,
+                    t_sec) == 6) ||
+         (sscanf(given,"%d-%d-%d-%d-%d-%d",
+                    t_year,
+                    t_month,
+                    t_day,
+                    t_hr,
+                    t_min,
+                    t_sec) == 6))
+
+    {
+        /* added sanity check on Jul-17-1999 */
+        if (*t_year < 0)
+            return(False);
+
+        if (*t_month < 1 || *t_month > 12)
+            return(False);
+
+        if (*t_day < 0 || *t_day > 31)
+            return(False);
+
+        if (*t_hr < 0 || *t_hr > 24)
+            return(False);
+
+        if (*t_min < 0 || *t_min > 59)
+            return(False);
+
+        if (*t_sec < 0 || *t_sec > 59)
+            return(False);
+
+        /* some jerk might provide a very large year to overflow */
+        if (*t_year > MAX_COUNTDOWN_YEAR)
+            return (False);
+    }
+
+     return (True);
+}
+
+
+/*
+** parse a buffer which conatints  integer1;integer2
+*/
+static int ParseTwoint(given,set_x,set_y )
 char
     *given;
 int 
-    *set_w, 
-    *set_h;
+    *set_x, 
+    *set_y;
 {
     int
         rc;
 
-    rc=sscanf(given, "%d;%d", set_w, set_h);
+    rc=sscanf(given, "%d;%d", set_x, set_y);
 
     if (rc != 2)
         return(False);
 
-    *set_w = AbsoluteValue(*set_w);
-    *set_h = AbsoluteValue(*set_h);
+    *set_x = AbsoluteValue(*set_x);
+    *set_y = AbsoluteValue(*set_y);
     return(True);
 }
 
@@ -668,12 +546,86 @@ char
                keyword, blurb, value);
     }
 
-    PrintHeader();
     StringImage(emsg);
     current_rc++;
     return(current_rc);
 }
 
+/*
+** returns True if OK, False otherwise
+*/
+static int parseAlignString(given, sets)
+char
+    *given;
+int
+    *sets;
+{
+    if (mystrncasecmp(given, "left", 4) == 0)
+    {
+        *sets = ALIGN_LEFT;
+        return (True);
+    }
+    else if (mystrncasecmp(given, "right", 5) == 0)
+    {
+        *sets = ALIGN_RIGHT;
+        return (True);
+    }
+    else if (mystrncasecmp(given, "center", 6) == 0)
+    {
+        *sets = ALIGN_CENTER;
+        return (True);
+    }
+    else if (mystrncasecmp(given,"topleft",7) == 0)
+    {
+        *sets=ALIGN_TOP_LEFT;
+        return (True);
+    }
+    else if (mystrncasecmp(given,"topcenter",9) == 0)
+    {
+        *sets=ALIGN_TOP_CENTER;
+        return (True);
+    }
+    else if (mystrncasecmp(given,"topright",8) == 0)
+    {
+        *sets=ALIGN_TOP_RIGHT;
+        return (True);
+    }
+    else if (mystrncasecmp(given,"midleft",7) == 0)
+    {
+        *sets=ALIGN_MID_LEFT;
+        return (True);
+    }
+    else if (mystrncasecmp(given,"midcenter",9) == 0)
+    {
+        *sets=ALIGN_MID_CENTER;
+        return (True);
+    }
+    else if (mystrncasecmp(given,"midright",8) == 0)
+    {
+        *sets=ALIGN_MID_RIGHT;
+        return (True);
+    }
+    else if (mystrncasecmp(given,"bottomleft",10) == 0)
+    {
+        *sets=ALIGN_BOTTOM_LEFT;
+        return (True);
+    }
+    else if (mystrncasecmp(given,"bottomcenter",12) == 0)
+    {
+        *sets=ALIGN_BOTTOM_CENTER;
+        return (True);
+    }
+    else if (mystrncasecmp(given,"bottomright",11) == 0)
+    {
+        *sets=ALIGN_BOTTOM_RIGHT;
+        return (True);
+    }
+    else
+        return (False);
+}
+
+
+
 int ParseQueryString(qs, digit_info, frame_info)
 char
     *qs;
@@ -712,9 +664,7 @@ FrameInfo
         value=mystrtok(p,PARAM_DELIMITERS);
         if (value == (char *) NULL)
         {
-            (void) sprintf(emsg,
-                "Parameter \"%.100s=\" requires a value\n", keyword);
-            PrintHeader();
+            (void) sprintf(emsg,"Parameter \"%.100s=\" requires a value\n", keyword);
             StringImage(emsg);
             rc++;
             break;
@@ -728,7 +678,7 @@ FrameInfo
         * parsed; this allows all errors to be shown to user in first pass.
         * Hence "rc" is incremented with each error found.
         ********************************************************************/
-        if (mystrcasecmp(keyword,"ft") == 0)
+        if (mystrncasecmp(keyword,"ft",2) == 0)
         {
             if (!ParseInteger(value,&dummy,0,255))
                 rc = ParseError(keyword,value,rc,
@@ -736,7 +686,7 @@ FrameInfo
             else
                 frame_info->width= (unsigned int) dummy;
         }
-        else if (mystrcasecmp(keyword,"cache") == 0)
+        else if (mystrncasecmp(keyword,"cache",3) == 0)
         {
             if (!ParseBool(value,&dummy))
                 rc=ParseError(keyword,value,rc,"non-Boolean");
@@ -744,7 +694,7 @@ FrameInfo
                 digit_info->cache=dummy;
         }
  
-        else if (mystrcasecmp(keyword,"expires") == 0)
+        else if (mystrncasecmp(keyword,"expires",3) == 0)
         {
             if (!ParseInteger(value,&dummy,0,604800))
                 rc=ParseError(keyword,value,rc,
@@ -752,7 +702,7 @@ FrameInfo
             else
                 digit_info->expires=dummy;
         }
-        else if (mystrcasecmp(keyword,"frgb") == 0)
+        else if (mystrncasecmp(keyword,"frgb",4) == 0)
         {
             if ( ! ParseRGB(value,
                 &(frame_info->matte_color.red),
@@ -761,14 +711,14 @@ FrameInfo
                 rc = ParseError(keyword,value,rc,"badly specified RGB");
 
         }
-        else if (mystrcasecmp(keyword,"chcolor") == 0)
+        else if (mystrncasecmp(keyword,"chcolor",7) == 0)
         {
             if (!ParseBool(value, &dummy))
                 rc = ParseError(keyword,value,rc,"non-Boolean");
             else
                 digit_info->replace_color=(unsigned int) dummy;
         }
-        else if (mystrcasecmp(keyword,"srgb") == 0)
+        else if (mystrncasecmp(keyword,"srgb",4) == 0)
         {
             if (!ParseRGB(value,
                 &(digit_info->opaque_red),
@@ -777,7 +727,7 @@ FrameInfo
                 rc=ParseError(keyword,value,rc,
                     "badly specified source RGB");
         }
-        else if (mystrcasecmp(keyword,"prgb") == 0)
+        else if (mystrncasecmp(keyword,"prgb",4) == 0)
         {
             if (!ParseRGB(value,
                 &(digit_info->pen_red),
@@ -788,14 +738,7 @@ FrameInfo
             else
                 digit_info->replace_color=True;
         }
-        else if (mystrcasecmp(keyword,"tr") == 0)
-        {
-            if (!ParseBool(value,&dummy))
-                rc = ParseError(keyword,value,rc,"non-Boolean");
-            else
-                digit_info->alpha=(unsigned int) dummy;
-        }
-        else if (mystrcasecmp(keyword, "trgb") == 0)
+        else if (mystrncasecmp(keyword, "trgb",4) == 0)
         {
             if ( ! ParseRGB(value,
                 &(digit_info->alpha_red),
@@ -804,40 +747,47 @@ FrameInfo
                 rc = ParseError(keyword,value,rc,"badly specified RGB");
             digit_info->alpha=True;
         }
-        else if (mystrcasecmp(keyword,"md") == 0)
+        else if (mystrncasecmp(keyword,"tr",2) == 0)
+        {
+            if (!ParseBool(value,&dummy))
+                rc = ParseError(keyword,value,rc,"non-Boolean");
+            else
+                digit_info->alpha=(unsigned int) dummy;
+        }
+        else if (mystrncasecmp(keyword,"md",2) == 0)
         {
             if (!ParseInteger(value,&(digit_info->maxdigits),5,MAX_DIGITS))
                 rc = ParseError(keyword,value,rc,"bad max digits");
         }
-        else if (mystrcasecmp(keyword,"pad") == 0)
+        else if (mystrncasecmp(keyword,"pad",3) == 0)
         {
             if (!ParseBool(value, &dummy))
                 rc = ParseError(keyword,value,rc,"non-Boolean");
             else
                 digit_info->leftpad=(unsigned int) dummy;
         }
-        else if (mystrcasecmp(keyword,"dd") == 0)
+        else if (mystrncasecmp(keyword,"dd",2) == 0)
         {
             /* safe copy */
             safeStrcpy(digit_info->ddhead,value,
                           sizeof(digit_info->ddhead)-1);
            Debug2("sizeof(digit_info->ddhead)=%d",sizeof(digit_info->ddhead),0);
         }
-        else if (mystrcasecmp(keyword,"st") == 0)
+        else if (mystrncasecmp(keyword,"st",2) == 0)
         {
             if (!ParseInteger(value, &dummy, 1,999999999))
                 rc = ParseError(keyword,value,rc,"bad starting");
             else
                 digit_info->st=(unsigned int) dummy;
         }
-        else if (mystrcasecmp( keyword, "sh") == 0)
+        else if (mystrncasecmp( keyword, "sh",2) == 0)
         {
             if (!ParseBool(value, &dummy))
                 rc = ParseError(keyword,value,rc,"non-Boolean");
             else
                 digit_info->show=(unsigned int) dummy;
         }
-        else if (mystrcasecmp(keyword,"istrip") == 0)
+        else if (mystrncasecmp(keyword,"istrip",6) == 0)
         {
             if (!ParseBool(value, &dummy))
                rc = ParseError(keyword,value,rc,"non-Boolean");
@@ -845,38 +795,47 @@ FrameInfo
                digit_info->use_strip=(unsigned int) dummy;
 
         }
-        else if (mystrcasecmp(keyword,"comma") == 0)
+        else if (mystrncasecmp(keyword,"comma",5) == 0)
         {
             if (!ParseBool(value, &dummy))
                rc = ParseError(keyword,value,rc,"non-Boolean");
             else
                digit_info->comma=(unsigned int) dummy;
         }
-        else if (mystrcasecmp(keyword,"nsegment") == 0)
+        else if (mystrncasecmp(keyword,"nsegment",8) == 0)
         {
             if (!ParseInteger(value, &(digit_info->nsegment), 1,999999999))
                 rc = ParseError(keyword,value,rc,"bad starting");
         }
-        else if (mystrcasecmp(keyword,"df") == 0)
+        else if (mystrncasecmp(keyword,"dformat",7) == 0)
+        {
+            if (!ParseDateFormat(value,&(digit_info->date_format)))
+                rc=ParseError(keyword,value,rc,"invalid");
+            /* if yyyy is thre, make full year on, mm Jan-27-1999 */
+            if (mystristr(value,"yyyy") != NULL)
+                digit_info->show_full_year=True;
+            
+        }
+        else if (mystrncasecmp(keyword,"df",2) == 0)
         {
             safeStrcpy(digit_info->datafile,value,
                            sizeof(digit_info->datafile)-1);
         }
-        else if (mystrcasecmp( keyword,"incr") == 0)
+        else if (mystrncasecmp( keyword,"incr",4) == 0)
         {
             if (!ParseBool(value, &dummy))
                 rc = ParseError(keyword,value,rc,"non-Boolean");
             else
                 digit_info->increment_counter=(unsigned int) dummy;
         }
-        else if (mystrcasecmp( keyword,"negate") == 0)
+        else if (mystrncasecmp( keyword,"negate",6) == 0)
         {
             if (!ParseBool(value, &dummy))
                 rc = ParseError(keyword,value,rc,"non-Boolean");
             else
                 digit_info->negate=(unsigned int) dummy;
         }
-        else if (mystrcasecmp( keyword,"wxh") == 0)
+        else if (mystrncasecmp( keyword,"wxh",3) == 0)
         {
             /*
             ** do nothing, this keyword is here just for backward
@@ -884,14 +843,15 @@ FrameInfo
             ** we determine digit dimensions automatically
             */
         }
-        else if (mystrcasecmp( keyword,"rotate") == 0)
+        else if (mystrncasecmp( keyword,"rotate",6) == 0)
         {
             if (!ParseBool(value, &dummy))
                 rc = ParseError(keyword,value,rc,"non-Boolean");
             else
                 digit_info->rotate=(unsigned int) dummy;
         }
-        else if (mystrcasecmp(keyword,"degrees") == 0)
+        else if ((mystrncasecmp(keyword,"degrees",7) == 0) ||
+                 (mystrncasecmp(keyword,"angle",5) == 0))
         {
             if (!ParseInteger(value,&(digit_info->rotate_degrees),90,360))
                 rc = ParseError(keyword,value,rc,
@@ -899,50 +859,132 @@ FrameInfo
             else
                 digit_info->rotate=True;
         }
-        else if (mystrcasecmp(keyword,"lit") == 0)
+        else if (mystrncasecmp(keyword,"lit",3) == 0)
         {
            if (!ParseLiteralString(value,digit_info->literal,100))
                 rc = ParseError(keyword,value,rc,"un-supported or oversized");
         }
-        else if (mystrcasecmp(keyword,"display") == 0)
+        else if (mystrncasecmp(keyword,"display",3) == 0)
         {
             if (!ParseOptionString(value,&(digit_info->display_type)))
                 rc=ParseError(keyword,value,rc,"unknown");
         }
-        else if (mystrcasecmp(keyword,"image") == 0)
+        else if (mystrncasecmp(keyword,"image",5) == 0)
         {
-            digit_info->display_type=SHOW_GIF_FILE;
+            /* 
+            ** this keyword does make display type image anymore. it's
+            ** possible that the user is trying to composite counter, clock
+            ** countdown with a base image
+            */
             safeStrcpy(digit_info->gif_file,value,
                     sizeof(digit_info->gif_file)-1);
         }
-        else if (mystrcasecmp(keyword,"dformat") == 0)
-        {
-            if (!ParseDateFormat(value,&(digit_info->date_format)))
-                rc=ParseError(keyword,value,rc,"invalid");
-        }
-        else if (mystrcasecmp(keyword,"tformat") == 0)
+        else if (mystrncasecmp(keyword,"tformat",7) == 0)
         {
             if (!ParseInteger(value,&(digit_info->time_format),12,24))
                 rc=ParseError(keyword,value,rc,
                     "badly specified time format, (use 12 or 24)");
         }
-        else if (mystrcasecmp(keyword,"script") == 0) /* for cgiwrap*/
+        else if (mystrncasecmp(keyword,"script",6) == 0) /* for cgiwrap*/
         {
             ;
         }
-        else if (mystrcasecmp(keyword,"user") == 0) /*for cgiwrap*/
+        else if (mystrncasecmp(keyword,"user",4) == 0) /*for cgiwrap*/
         {
             ;
         }
-        else if (mystrcasecmp(keyword,"timezone") == 0)
+        else if (mystrncasecmp(keyword,"timezone",8) == 0)
         {
             safeStrcpy(digit_info->time_z,value,
                     sizeof(digit_info->time_z)-1);
         }
+        else if (mystrncasecmp(keyword,"reload",6) == 0)
+        {
+            if (!ParseBool(value, &dummy))
+                rc = ParseError(keyword,value,rc,"non-Boolean");
+            else
+                digit_info->user_reload=(unsigned int) dummy;
+        }
+        else if (mystrncasecmp(keyword,"cdf",3) == 0)
+        {
+            if (! ParseCountdown(value,
+                        &(digit_info->f_year),
+                        &(digit_info->f_mon),
+                        &(digit_info->f_day),
+                        &(digit_info->f_hr),
+                        &(digit_info->f_min),
+                        &(digit_info->f_sec)))
+            {
+                   rc=ParseError(keyword,value,rc,
+                           "badly specified cowntdown from");
+            }
+            digit_info->display_type=SHOW_COUNTDOWN;
+        }
+        else if ((mystrncasecmp(keyword,"cdt",3) == 0) ||
+                 (mystrncasecmp(keyword,"target",6) == 0))
+        {
+            if (! ParseCountdown(value,
+                        &(digit_info->t_year),
+                        &(digit_info->t_mon),
+                        &(digit_info->t_day),
+                        &(digit_info->t_hr),
+                        &(digit_info->t_min),
+                        &(digit_info->t_sec)))
+            {
+                   rc=ParseError(keyword,value,rc,
+                           "badly specified cowntdown target");
+            }
+            digit_info->display_type=SHOW_COUNTDOWN;
+        }
+        else if (mystrncasecmp(keyword,"sfd",3) == 0)
+        {
+            /* show from date of countdown below the image */
+            if (!ParseBool(value, &dummy))
+               rc = ParseError(keyword,value,rc,"non-Boolean");
+            else
+               digit_info->show_from_date=(int) dummy;
+        }
+        else if (mystrncasecmp(keyword,"sdhms",5) == 0)
+        {
+            if (!ParseBool(value,&dummy))
+                rc=ParseError(keyword,value,rc,"non-Boolean");
+            else
+                digit_info->show_dhms=(int) dummy;
+        }
+        else if (mystrncasecmp(keyword,"offset",6) == 0)
+        {
+            if (!ParseInteger(value,&dummy,0,999999999))
+                rc = ParseError(keyword,value,rc,
+                    "badly specified frame offset");
+            else
+                digit_info->offset= (unsigned int) dummy;
+        }
+        else if (mystrncasecmp(keyword,"xy",2) == 0)
+        {
+            if (! ParseTwoint(value,
+                        &(digit_info->upl_x),
+                        &digit_info->upl_y))
+            {
+                   rc=ParseError(keyword,value,rc,"badly specified sixy");
+            }
+        }
+        else if (mystrncasecmp(keyword,"align",5) == 0)
+        {
+            if (!parseAlignString(value,&(digit_info->alignment)))
+                rc=ParseError(keyword,value,rc,"unknown");
+            digit_info->align=True;
+        }
+        else if (mystrncasecmp(keyword,"fy",2) == 0)
+        {
+            /* show full year instead of 2 digit year */
+            if (!ParseBool(value, &dummy))
+                rc = ParseError(keyword,value,rc,"non-Boolean");
+            else
+                digit_info->show_full_year=(unsigned int) dummy;
+        }
         else 
         {
             sprintf(emsg, "Bad keyword \"%.100s\"", keyword);
-            PrintHeader();
             StringImage(emsg);
             rc++;
         }
@@ -1143,103 +1185,6 @@ struct	tm	*tm;
 
 #endif /* !HAVE_MKTIME */
 
-/*
- *  checkTimezone - parses the GMT+-hhmm and returns the time in seconds
- *
- *  RCS:
- *      $Revision: 2.3 $
- *      $Date: 1996/05/03 02:20:22 $
- *
- *  Security:
- *      Unclassified
- *
- *  Description:
- *      This funcions parses a string like GMT+-hhmm and returns the time
- *      in seconds.
- *
- *  Input Parameters:
- *      type    identifier  description
- *
- *      char    str[]
- *          str holds a string like GMT+-hhmm
- *
- *  Output Parameters:
- *      type    identifier  description
- *
- *      text
- *
- *  Return Values:
- *      value   description
- *      int     diff
- *          hhmm is converted to seconds before returning. the value will
- *          be positive or negative based on GMT+ or GMT-
- *
- *  Side Effects:
- *      text
- *
- *  Limitations and Comments:
- *      text
- *
- *  Development History:
- *      who                 when        why
- *      muquit@semcor.com   26-Mar-96   first cut
- */
-
-
-long checkTimezone(str)
-char
-    str[];
-{
-    int
-        hr,
-        min;
-    int
-        d;
-    int
-        counter=0;
-
-    char
-        buf[10];
-
-    long
-        diff=0L;
-
-    int
-        i;
-
-    *buf='\0';
-    hr=0;
-    min=0;
-    
-
-    if (*str != '\0')
-    {
-        if (strlen(str) == 8)
-        {
-            if ((str[3] == '-') || (str[3] == '+'))
-            {
-                (void) strcpy(buf,&str[4]);   
-                for (i=0; i < strlen(buf); i++)
-                {
-                    d=buf[i];
-                    if ((d >= '0') && (d <= '9'))
-                    {
-                        if (counter < 2)
-                            hr=hr*10 +(d-'0');
-                        else
-                            min=min*10+(d-'0');
-                        counter++;
-                    }
-                }
-                diff=(hr*3600)+min*60;
-                if (str[3] == '-')
-                    diff = -(diff);
-            }
-        }
-    }
-    return (diff);
-}
-
 /*
 ** parse literal string
 ** 04/23/96
@@ -1258,6 +1203,10 @@ int
              (*given == ':')  ||
              (*given == ',')  ||
              (*given == '-')  ||
+             (*given == 'D')  ||
+             (*given == 'H')  ||
+             (*given == 'M')  ||
+             (*given == 'S')  ||
              (*given == 'a')  ||
              (*given == 'A')  ||
              (*given == 'p')  ||
@@ -1273,121 +1222,6 @@ int
 }
 
 
-/* writes the error message image and exits with 0 */
-static void printConfigError(err_type)
-int
-    err_type;
-{
-
-    char
-        tmpbuf[1024];
-
-    switch (err_type)
-    {
-        case ConfigOpenFailed:
-        {
-            *tmpbuf='\0';
-            (void) sprintf (tmpbuf,
-                "Failed to open configuration file: \"%s"
-                DIR_SEPARATOR "%s\"",
-                    CONFIG_DIR,CONFIG_FILE);
-            Warning (tmpbuf);
-            PrintHeader ();
-            StringImage (tmpbuf);
-            exit(0);
-            break;
-        }
-
-        case NoAutofcBlock:
-        {
-            *tmpbuf='\0';
-            (void) sprintf (tmpbuf,
-                "No auto file creation block in conf file: \"%s" 
-                DIR_SEPARATOR "%s\"",
-                    CONFIG_DIR,CONFIG_FILE);
-            Warning(tmpbuf);
-            PrintHeader ();
-            StringImage(tmpbuf);
-            exit(0);
-            break;
-        }
-
-        case NoStrictBlock:
-        {
-            *tmpbuf='\0';
-            (void) sprintf (tmpbuf,
-                "No Strict mode block in conf file: \"%s"
-                DIR_SEPARATOR "%s\"",
-                    CONFIG_DIR,CONFIG_FILE);
-            Warning(tmpbuf);
-            PrintHeader ();
-            StringImage(tmpbuf);
-            exit(0);
-            break;
-        }
-        case NoIgnoreHostsBlock:
-        {
-            *tmpbuf='\0';
-            (void) sprintf (tmpbuf,
-                "No IgnoreHosts Block in the configuration file: \"%s"
-                DIR_SEPARATOR "%s\"",
-                    CONFIG_DIR,CONFIG_FILE);
-
-            Warning(tmpbuf);
-            PrintHeader ();
-            StringImage(tmpbuf);
-            exit(0);
-        }
-
-        case NoRefhBlock:
-        {
-            *tmpbuf='\0';
-            (void) sprintf(tmpbuf,"Compiled with -DACCESS_AUTH,but no such block in configuration file!");
-            Warning(tmpbuf);
-            PrintHeader ();
-            StringImage(tmpbuf);
-            exit(1);
-        }
-
-        case UnpexpectedEof:
-        {
-            *tmpbuf='\0';
-            (void) sprintf (tmpbuf,
-            "Unexpected EOF in configuration file: \"%s"
-            DIR_SEPARATOR "%s\"",
-                CONFIG_DIR,CONFIG_FILE);
-            Warning(tmpbuf);
-            PrintHeader ();
-            StringImage(tmpbuf);
-            exit(0);
-        }
-
-        /* 
-        ** NoRgbMappingBlock was not handled, thanks to 
-        ** phil@pottsoft.daemon.co.uk
-        ** Nov-26-1997
-        */
-
-        case NoRgbMappingBlock:
-        {
-            *tmpbuf='\0';
-            (void) sprintf (tmpbuf,"No RGB Mapping block in config file!");
-            Warning(tmpbuf);
-            PrintHeader();
-            StringImage(tmpbuf);
-            exit(0);
-        }
-
-        default:
-        {
-            Warning("unknown error code while reading config file");
-            PrintHeader();
-            StringImage("unknown error code while reading config file");
-            exit(0);
-        }
-    }   /* switch */
-}
-
 /*
 **  initImageParams()
 **  initialized digit_info and frame_info
@@ -1424,7 +1258,8 @@ FrameInfo
     frame_info->matte_color.green=(unsigned char) DEFAULT_FRAME_GREEN;
     frame_info->matte_color.blue=(unsigned char) DEFAULT_FRAME_BLUE;
 
-    digit_info->ub=False;
+    digit_info->pdigits=(char *) NULL;
+
     digit_info->replace_color=False;
     digit_info->cache=1;        /* cache */
     digit_info->expires=0;      /* expires right away*/
@@ -1452,7 +1287,6 @@ FrameInfo
     digit_info->rotate_degrees=270;
     safeStrcpy(digit_info->datafile,DEFAULT_DATAFILE,
             sizeof(digit_info->datafile)-1);
-    Debug2("xxxxxxxxx %s",digit_info->datafile,0);
     Debug2("sizeof(digit_info->datafile)=%d",sizeof(digit_info->datafile),0);
     safeStrcpy(digit_info->ddhead,DEFAULT_DIGITS_STYLE,
             sizeof(digit_info->ddhead)-1);
@@ -1467,7 +1301,44 @@ FrameInfo
     digit_info->comma=False;
     digit_info->nsegment=15;
     *digit_info->gif_file='\0';
-    Debug2("xxxxxxxxx %s",digit_info->datafile,0);
+    digit_info->user_reload=1;
+
+    /* dec-13-1998, mm */
+    /* default countdown targe is Year 2000 */
+    digit_info->t_year=2000;
+    digit_info->t_mon=1;
+    digit_info->t_day=1;
+    digit_info->t_hr=0;
+    digit_info->t_min=0;
+    digit_info->t_sec=0;
+
+    /* 
+    ** default countdown from values, all negative, so the user must specify
+    ** them
+    */
+    digit_info->f_year=(-1);
+    digit_info->f_mon=(-1);
+    digit_info->f_day=(-1);
+    digit_info->f_hr=(-1);
+    digit_info->f_min=(-1);
+    digit_info->f_sec=(-1);
+
+
+
+    /* dont' show from below the countdown time by default */
+    digit_info->show_from_date=0;
+
+    /* show day hr min sec in countdown by default */
+    digit_info->show_dhms=1;
+
+    digit_info->upl_x=(-1);
+    digit_info->upl_y=(-1);
+
+    digit_info->show_full_year=False;
+
+    digit_info->align=False;
+    digit_info->alignment=ALIGN_TOP_CENTER;
+    digit_info->offset=2;
 
 }
 
