@@ -41,6 +41,14 @@
 #define SYM_PREFIX ""
 #endif
 
+static char *month_name[]=
+{
+    "Jan", "Feb", "Mar", "Apr",
+    "May", "Jun", "Jul", "August",
+    "Sep", "Octr","Nov", "Dec"
+};
+
+
 static int howmanyCommas _Declare((char *));
 static unsigned long my_inet_addr _Declare ((char *));
 
@@ -94,7 +102,7 @@ char
 
 
 /*
-**  Commaize()
+**  Commaize_buggy()
 **  add a comma after every 3rd digit from right
 **
 **  RCS
@@ -110,7 +118,9 @@ char
 **      buf is modified if it is atleast 4 character long
 **
 **  Limitations and Comments:
-**      none
+************************************************************************
+**     ********** Don't use this, it's a bug!, mm, may-26-1999 *****
+************************************************************************
 **
 **  Development History:
 **      who                  when       why
@@ -119,7 +129,7 @@ char
 */
 
 
-void Commaize(buf)
+void Commaize_buggy(buf)
 char
     *buf;
 {
@@ -179,6 +189,68 @@ char
 }
 
 
+/*
+**  Commaize()
+**  add a comma after every 3rd digit from right
+**
+**  RCS
+**      $Revision$
+**      $Date$
+**  Return Values:
+**      none
+**
+**  Parameters:
+**      buf     buf to modify
+**
+**  Side Effects:
+**      buf is modified
+**
+**  Limitations and Comments:
+**      buf must have enough space to hold the extra , characters
+**
+**  Development History:
+**      who                  when       why
+**      ma_muquit@fccc.edu   no idea    first cut
+**                           Oct-18-97  added dynamic buffer 
+**                           May-26-1999 rewrote as the old version has
+**                           memory overstepping bug. Code adapted from
+**                           C-Snippets, file commaflt.c by Bruce Wedding and
+**                           and Kurt Kuzba
+*/
+
+void Commaize(buf)
+char
+    *buf;
+{
+    char
+        *pbuf=(char *) NULL;
+
+    int
+        bf=0,
+        cm=0,
+        tm=0;
+
+    if (*buf != '\0')
+    {
+        pbuf=mystrdup(buf);
+        if (pbuf == (char *) NULL)
+            return; /* malloc failed, return quitely */
+
+        reverseString(pbuf);
+        while ((buf[bf++]=pbuf[tm++]) != 0)
+        {
+            if(++cm % 3 == 0 && pbuf[tm])
+                buf[bf++]=',';
+        }
+        if (pbuf)
+            (void) free((char *) pbuf);
+
+        reverseString(buf);
+    }
+}
+
+
+
 /*
 ** print the Content-type MIME header
 */
@@ -302,11 +374,10 @@ int
     char
         *ptr;
 
-    ptr=(char *) malloc(size);
+    ptr=(char *) malloc(size*sizeof(char));
 
     if (ptr == NULL)
     {
-        PrintHeader();
         StringImage("Memory allocation problem!");
         exit(0);
     }
@@ -388,7 +459,6 @@ int
     /*
     if ( NULL == from_str )
     {
-        PrintHeader();
         StringImage("Source buffer is NULL in safeStrcpy()!");
         exit(0);
     }
@@ -454,14 +524,12 @@ int
 
    if ( NULL == str )
    {
-       PrintHeader();
        StringImage("Source buffer is NULL in safeStrcat()!");
        exit(0);
    }
 
    if (strlen(str) >= ssc_size - ssc_length)
    {
-       PrintHeader();
        StringImage("buffer overflow detected! aborting");
        exit(0);
    }
@@ -478,14 +546,20 @@ int
    /* now copy */
    (void) mystrncat(to_str,str,copy_length);
 
+   /*
    to_str[ssc_length + copy_length] = '\0';
+   */
 }
 
 
 
 /* case insensitive version of ANSI strstr() */
 /* from swish package by kevin h , kevin called it lstrstr()*/
-char *stristr(s, t)
+/*
+** s - hay stack
+** t - needle
+*/
+char *mystristr(s,t)
 char
     *s;
 char
@@ -501,7 +575,7 @@ int
     for (i = 0; s[i]; i++) 
     {
         for (j = 0, l = k = i; s[k] && t[j] &&
-        tolower(s[k]) == tolower(t[j]); j++, k++)
+            tolower(s[k]) == tolower(t[j]); j++, k++)
             ;
         if (t[j] == '\0')
             return s + l;
@@ -544,6 +618,7 @@ int authRemhost()
 
     char
         *prem_refh=(char *) NULL,
+        szbuf[BUFSIZ],
         szHttp_ref[1024],
         szRhost[1024];
 
@@ -555,9 +630,6 @@ int authRemhost()
         *ipaddr;
 #endif
 
-    Sllist
-        *llp;
-
     *szHttp_ref='\0';
     *szRhost='\0';
 
@@ -584,39 +656,46 @@ int authRemhost()
                 ipaddr = inet_ntoa(*(struct in_addr *) he->h_addr);
              }
 #endif
-            if (Grhost > 0)
-            {
 
-                for (llp=g_refhost;llp;llp=llp->next)
-                {
-                    rc=isinname(szRhost,llp->item);
+            rc=hostAuthorized(szRhost);
 #ifdef USE_DNS
-                    if (!rc && ipaddr) 
-                    {
-                        rc=isinname(ipaddr, llp->item);
-                    }
-#endif
-
-                    if (rc == 1)    /* allowed */
-                        return(1);
-                }
+            if (!rc && ipaddr)
+            {
+                rc=hostAuthorized(ipaddr);
             }
-            if (rc == 0)
+#endif /* USE_DNS */
+            if (rc == True)     /* Authorized */
+                return (True);
+            
+            if (rc == False)
             {
                 *szHttp_ref='\0';
                 if (strlen(szRhost) + 27 > sizeof(szHttp_ref))
                 {
-                    (void) sprintf(szHttp_ref,"Host: \"(name too long)\" is not authorized");
+                    (void) sprintf(szHttp_ref,
+                                "Host: \"(name too long)\" is not authorized");
                 }
                 else
                 {
                     (void) sprintf(szHttp_ref,"Host: \"%s\" is not authorized",
                                    szRhost);
+                    Warning(szHttp_ref);
+                    StringImage(szHttp_ref);
+
+                    /*
+                    ** log the offending page too. Oops, the code was
+                    ** buggy! may be I was sleepy? Someone sent the fix.
+                    ** will put the name in ChangeLog.
+                    ** Sep-07-2000
+                    */
+                    if (strlen(prem_refh) + 16 < sizeof(szbuf))
+                    {
+                        (void) sprintf(szbuf,"Referer page: %s\n",
+                                       prem_refh);
+                        Warning(szbuf);
+                    }
                 }
-                Warning(szHttp_ref);
 
-                PrintHeader();
-                StringImage(szHttp_ref);
                 exit(0);
             }
         }
@@ -695,7 +774,7 @@ char
     {
         if (lastchar == '*')
         {
-            if ((char *) stristr(string, tempmask))
+            if ((char *) mystristr(string, tempmask))
             {
                 free(tempmask);
                 return 1;
@@ -703,7 +782,7 @@ char
         }
         else
         {
-            if ((char *) stristr(string, tempmask) ==
+            if ((char *) mystristr(string, tempmask) ==
                  string + strlen(string) - strlen(tempmask))
             {
                 free(tempmask);
@@ -713,7 +792,7 @@ char
     }
     else if (lastchar == '*')
     {
-        if ((char *) stristr(string, tempmask) == string)
+        if ((char *) mystristr(string, tempmask) == string)
         {
             free(tempmask);
             return 1;
@@ -768,23 +847,41 @@ char
     char
         buf[1024];
 
+    /*
+    ** if configured not to log messages, just return
+    */
+    if (getcfgLogErrmsg() == False)
+        return;
+
     *buf='\0';
+
 #ifdef SYS_WIN32
     (void) sprintf(buf,"%s%s/%s",g_sZ_vbase,LOG_DIR,LOG_FILE);
 #else
     (void) sprintf(buf,"%s" DIR_SEPARATOR "%s",LOG_DIR,LOG_FILE);
 #endif
 
-    times = GetTime();
-    fp = fopen(buf, "a");
+    times=GetTime();
 
+
+    fp = fopen(buf, "a");
     if (fp == (FILE *) NULL)
     {
-        (void) fprintf (stderr,"[%s] Count %s: Could not open CountLog file %s"
-                        DIR_SEPARATOR "%s\n ",times, Version, LOG_DIR,LOG_FILE);
-        fp = stderr;
+        /*
+        ** netscape web server merges stderr and stdout together, so if you'r
+        ** using netscape server, you're busted, you not going to know why
+        ** the program is not working. If you use apache, you will see error
+        ** messages in the server error log
+        ** --play safe, just return quietly (mm, jan-21-1999)
+        */
+/*
+(void) fprintf (stderr,"[%s] Count %s: Could not open CountLog file %s\n",
+                times,Version,buf);
+*/
+        return;
     }
-        (void) fprintf (fp,"[%s] Count %s: %s\n", times, Version,message);
+    (void) fprintf (fp,"[%s] Count %s: %s\n", times, Version,message);
+
     if (fp != stderr)
         (void) fclose (fp);
 }
@@ -897,6 +994,40 @@ cont:
 	  /* NOTREACHED */
 }
 
+/*
+** muquit, first cut: Jan-21-1999
+** was ASI definition. fixed. apr-26-1999
+*/
+int mystrncasecmp(s1,s2,n)
+char
+    *s1,
+    *s2;
+int
+    n;
+{
+    register char
+        *scan1,
+        *scan2;
+
+    int
+        count;
+
+    scan1=s1;
+    scan2=s2;
+    count=n;
+
+    while (--count >= 0 && *scan1 != '\0' && tolower(*scan1) == tolower(*scan2))
+    {
+        scan1++;
+        scan2++;
+    }
+    if (count < 0)
+        return (0);
+
+
+    return(tolower(*scan1) - tolower(*scan2));
+}
+
 
 /*
 ** found somewhere in the net
@@ -1080,7 +1211,6 @@ char *s;
 char *getdateGMT()
 {
     time_t
-        gmt_time_t,
         t_time_t;
 
     struct tm
@@ -1147,7 +1277,6 @@ int
 
 {
     time_t
-        gmt_time_t,
         t_time_t;
 
     struct tm
@@ -1203,11 +1332,15 @@ char
     if (string == (char *) NULL)
         return ((char *) NULL);
 
-    tmp = (char *) malloc ((int) strlen(string) + 1);
+    /*
+    **  malloc twice as much memory as a work around for something elsewhere 
+    **  muquit, may-26-1999
+    */
+    
+    tmp = (char *) malloc ((int) strlen(string)*2 );
 
     if (tmp == (char *) NULL)
     {
-        PrintHeader();
         StringImage("memory allocation problem mystrdup()");
         exit(0);
         return ((char *) NULL);
@@ -1289,3 +1422,408 @@ int
 
     return (dst);
 }
+
+/*
+ * strncpy - copy at most n characters of string src to dst
+ * adapted from henry's stringlib
+ */
+char *              /* dst */
+mystrncpy(dst,src,n)
+char
+    *dst,
+    *src;
+int
+    n;
+{
+    register char
+        *dscan;
+    register char
+        *sscan;
+    register int
+        count;
+
+    dscan = dst;
+    sscan = src;
+    count = n;
+    while (--count >= 0 && (*dscan++ = *sscan++) != '\0')
+        continue;
+    while (--count >= 0)
+        *dscan++ = '\0';
+    return(dst);
+}
+
+
+
+
+/*
+**  allocCountdown()
+**  allocate memory for the Countdown struct and initializes the members
+**
+**  Parameters:
+**      none
+**
+**  Return Values:
+**      Countdown   *       if succeeds
+**      NULL                if fails
+**
+**  Limitations and Comments:
+**      none
+**
+**  Development History:
+**      who                  when           why
+**      ma_muquit@fccc.edu   May-10-1998    first cut
+*/
+
+Countdown *allocCountdown()
+{
+    Countdown
+        *cd;
+
+    cd=(Countdown *) malloc(sizeof(Countdown));
+    if (cd == (Countdown *) NULL)
+        return ((Countdown *) NULL);
+
+    cd->days=0;
+    cd->hours=0;
+    cd->minutes=0;
+    cd->seconds=0;
+
+    return (cd);
+}
+
+/*
+**  calculate countdown time from a specific time and returns a pointer
+**  to the struct Countdown
+**
+**  Parameters:
+**      time_t  from_sec    From time in seconds
+**
+**  Return Values:
+**      pointer to struct Countdown in success
+**      NULL    in failure
+**
+**  Limitations and Comments:
+**      none
+**
+**
+**  Development History:
+**      who                  when           why
+**      ma_muquit@fccc.edu   May-10-1998    first cut for EJV
+**                           Dec-13-1998    in Count 2.4
+*/
+Countdown *countDown(t_year,t_mon,t_day,t_hr,t_min,t_sec,t1)
+int
+    t_year,
+    t_mon,
+    t_day,
+    t_hr,
+    t_min,
+    t_sec;
+
+time_t
+    t1;
+{
+    Countdown
+        *cdown=(Countdown *) NULL;
+
+    struct tm
+        tm;
+
+    time_t
+        t2,
+        tdiff;
+
+    cdown=allocCountdown();
+    if (cdown == NULL)
+        return ((Countdown *) NULL);
+
+    tm.tm_year=t_year-1900;        /* Year Since 1900 */
+    tm.tm_mday=t_day;
+    tm.tm_hour=t_hr;
+    tm.tm_min=t_min;
+    tm.tm_sec=t_sec;
+    tm.tm_mon=t_mon-1;
+    tm.tm_isdst=(-1);
+    t2=mktime(&tm);
+
+    tdiff=t2-t1;
+
+    cdown->days=tdiff/(60*60*24);
+    cdown->hours=(tdiff % (60*60*24))/3600;
+    cdown->minutes=(tdiff % 3600)/60;
+    cdown->seconds=(tdiff % 60);
+
+    return (cdown);
+}
+
+
+/*
+ *  checkTimezone - parses the GMT+-hhmm and returns the time in seconds
+ *
+ *  RCS:
+ *      $Revision: 2.3 $
+ *      $Date: 1996/05/03 02:20:22 $
+ *
+ *  Security:
+ *      Unclassified
+ *
+ *  Description:
+ *      This funcions parses a string like GMT+-hhmm and returns the time
+ *      in seconds.
+ *
+ *  Input Parameters:
+ *      type    identifier  description
+ *
+ *      char    str[]
+ *          str holds a string like GMT+-hhmm
+ *
+ *  Output Parameters:
+ *      type    identifier  description
+ *
+ *      text
+ *
+ *  Return Values:
+ *      value   description
+ *      int     diff
+ *          hhmm is converted to seconds before returning. the value will
+ *          be positive or negative based on GMT+ or GMT-
+ *
+ *  Side Effects:
+ *      text
+ *
+ *  Limitations and Comments:
+ *      text
+ *
+ *  Development History:
+ *      who                 when        why
+ *      muquit@semcor.com   26-Mar-96   first cut
+ */
+
+
+long checkTimezone(str)
+char
+    str[];
+{
+    int
+        hr,
+        min;
+    int
+        d;
+    int
+        counter=0;
+
+    char
+        buf[10];
+
+    long
+        diff=0L;
+
+    int
+        i;
+
+    *buf='\0';
+    hr=0;
+    min=0;
+    
+
+    if (*str != '\0')
+    {
+        if (strlen(str) == 8)
+        {
+            if ((str[3] == '-') || (str[3] == '+'))
+            {
+                (void) strcpy(buf,&str[4]);   
+                for (i=0; i < strlen(buf); i++)
+                {
+                    d=buf[i];
+                    if ((d >= '0') && (d <= '9'))
+                    {
+                        if (counter < 2)
+                            hr=hr*10 +(d-'0');
+                        else
+                            min=min*10+(d-'0');
+                        counter++;
+                    }
+                }
+                diff=(hr*3600)+min*60;
+                if (str[3] == '-')
+                    diff = -(diff);
+            }
+        }
+    }
+    return (diff);
+}
+/*
+**  getLocalTime()
+**  calculate time like: Mon-Day-Year hr:min:sec AM/PM GMT+xyzw
+**  Parameters:
+**  DigitInfo   *digitinfo  
+**  int add_from            if 1, add From: before the time string
+**
+**  Return Values:
+**  char *time  on success
+**  NULL on failure
+**
+**  Limitations and Comments:
+**  returns pointer to a malloc'd space, caller is responsible to free it.
+**  If gmtime() is not available in the system, it will call **  localtime() 
+**  instead of gmtime()
+**
+**  Development History:
+**      who                  when           why
+**      ma_muquit@fccc.edu   Jan-24-1999    first cut
+**                           Jul-17-1999    first arg is digit_info now
+*/
+
+char  *getLocalTime(digit_info,add_from)
+DigitInfo
+    *digit_info;
+int
+    add_from;
+{
+    char
+        *timezone_str,
+        *s=(char *) NULL;
+
+    struct tm
+        from_tm,
+        *ptm;
+
+    time_t
+        now;
+
+    long
+        diff;
+
+    timezone_str=digit_info->time_z;
+
+    /* if from values specified, calculate time_t for them -- mm,jul17,99 */
+    if (digit_info->f_year >  0 &&
+        digit_info->f_mon  >  0 &&
+        digit_info->f_day  >  0 &&
+        digit_info->f_hr   >= 0 &&
+        digit_info->f_min  >= 0 &&
+        digit_info->f_sec  >= 0)
+    {
+        from_tm.tm_year=digit_info->f_year - 1900;
+        from_tm.tm_mon=digit_info->f_mon - 1;
+        from_tm.tm_mday=digit_info->f_day;
+        from_tm.tm_hour=digit_info->f_hr;
+        from_tm.tm_min=digit_info->f_min;
+        from_tm.tm_sec=digit_info->f_sec;
+        from_tm.tm_isdst=(-1);
+
+        now=mktime(&from_tm);
+    }
+    else
+    {
+        /* local time */
+        now=time(NULL);
+    }
+
+    if (timezone_str == NULL || *timezone_str == '\0')
+    {
+        ptm=localtime(&now);
+    }
+    else
+    {
+        diff=checkTimezone(timezone_str);
+        now += diff;
+#ifdef HAVE_GMTIME
+        ptm=gmtime(&now);
+#else
+        ptm=localtime(&now);
+#endif /* HAVE_GMTIME */
+    }
+    s=cMalloc(512);
+    if (add_from)
+    {
+        if (timezone_str == NULL || *timezone_str == '\0')
+        {
+            (void) sprintf(s,"%s %s %d %d, %d:%d:%d",
+            "From:", month_name[ptm->tm_mon], ptm->tm_mday, 1900+ptm->tm_year,
+            ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
+        }
+        else
+            (void) sprintf(s,"%s %s %d %d, %d:%d:%d %s",
+            "From:", month_name[ptm->tm_mon], ptm->tm_mday, 1900+ptm->tm_year,
+            ptm->tm_hour, ptm->tm_min, ptm->tm_sec,timezone_str);
+    }
+    else
+    {
+        if (timezone_str == NULL || *timezone_str == '\0')
+        {
+            (void) sprintf(s,"%s %d %d, %d:%d:%d",
+            month_name[ptm->tm_mon], ptm->tm_mday, 1900+ptm->tm_year,
+            ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
+        }
+        else
+        {
+            (void) sprintf(s,"%s %d %d, %d:%d:%d %s",
+            month_name[ptm->tm_mon], ptm->tm_mday, 1900+ptm->tm_year,
+            ptm->tm_hour, ptm->tm_min, ptm->tm_sec,timezone_str);
+        }
+    }
+
+    return (s);
+}
+
+void makeGIFPath(path,path_len,base,digit_info)
+char
+    *path;
+
+int
+    path_len;
+
+char
+    *base;
+
+DigitInfo
+    *digit_info;
+{
+            /* make the gif filepath*/
+            safeStrcpy(path,base,path_len-1);
+#ifndef __VMS
+            safeStrcat(path,"/",path_len-1,path_len,strlen(path));
+#endif
+
+            safeStrcat(path,digit_info->ddhead,path_len-1,
+                    path_len,strlen(path));
+#ifndef __VMS
+            safeStrcat(path,"/",path_len-1,path_len, strlen(path));
+#else
+/*          For VMS add a ] before adding name of .gif file  */
+            safeStrcat(path,"]",path_len-1,path_len,
+                    strlen(path));
+#endif
+            safeStrcat(path,digit_info->gif_file,path_len-1,
+                    path_len,strlen(path));
+}
+
+
+/*
+** borrowed from c-snippets STRREV.C, public domain by Bob Stout
+** The name of the function was stttev()
+** -- muquit, May-26-1999
+** Comment: str is modified
+*/
+char *reverseString(str)
+char
+    *str;
+{
+    char
+        *p1,
+        *p2;
+
+    if (! str || ! *str)
+        return str;
+
+    for (p1=str,p2 =str+strlen(str)-1; p2 > p1; ++p1, --p2)
+    {
+        *p1 ^= *p2;
+        *p2 ^= *p1;
+        *p1 ^= *p2;
+    }
+    return (str);
+}
+
