@@ -8,14 +8,15 @@
 
 VER=`cat ../VERSION`
 DVER=`cat ../VERSION|sed 's/\./_/g'`
-PROG="testcount-sh"
-if [ -f $PROG ]; then
-    echo "$PROG"
-    $PROG
-else
-    echo "NO $PROG exists, exiting"
-    exit 0
-fi
+
+##PROG="./testcount-sh"
+##if [ -f $PROG ]; then
+##   echo "$PROG"
+##    $PROG
+##else
+##   echo "NO $PROG exists, exiting"
+##    exit 0
+##fi
 
 EXF="Count$DVER-ex.html"
 DOCDIR="../docs"
